// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


// GLUIView.cpp : implementation file
//

#include "StdAfx.h"
#include "GLView.h"

// The following are default implementations for the functions you will probably overload...


void 
CGLView::
OnCreateGL()
{
	// enable zbuffer
	glEnable(GL_DEPTH_TEST);
}


void 
CGLView::
OnSizeGL(int cx, int cy)
{
	// set correspondence between window and OGL viewport
	glViewport(0,0,cx,cy);

	// update the camera
	glPushMatrix();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(40.0, double(cx)/double(cy), 0.1f, 10.0f);
	glTranslatef(0.0f, 0.0f, -4.f);
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}


void 
CGLView::OnDrawGL()
{
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	// draw carthesian axes
	glBegin(GL_LINES);
	// red x axis
	glColor3f(1.f,0.f,0.f);
	glVertex3f(0.0f,0.0f,0.0f);
	glVertex3f(1.0f,0.0f,0.0f);
	glVertex3f(1.0f,0.0f,0.0f);
	glVertex3f(0.9f,0.1f,0.0f);
	glVertex3f(1.0f,0.0f,0.0f);
	glVertex3f(0.9f,-0.1f,0.0f);
	// green y axis
	glColor3f(0.f,1.f,0.f);
	glVertex3f(0.0f,0.0f,0.0f);
	glVertex3f(0.0f,1.0f,0.0f);
	glVertex3f(0.0f,1.0f,0.0f);
	glVertex3f(0.1f,0.9f,0.0f);
	glVertex3f(0.0f,1.0f,0.0f);
	glVertex3f(-0.1f,0.9f,0.0f);
	// blue z axis
	glColor3f(0.f,0.f,1.f);
	glVertex3f(0.0f,0.0f,0.0f);
	glVertex3f(0.0f,0.0f,1.0f);
	glVertex3f(0.0f,0.0f,1.0f);
	glVertex3f(0.0f,0.1f,0.9f);
	glVertex3f(0.0f,0.0f,1.0f);
	glVertex3f(0.0f,-0.1f,0.9f);
	glEnd();
}



// These are necessary...


void
CGLView::BeginGLCalls()
{
	wglMakeCurrent(m_CDC->GetSafeHdc(), m_hGLRC);
}


void 
CGLView::EndGLCalls()
{
	wglMakeCurrent(NULL,NULL);
}

    

void CGLView::BeginGLOverlayCalls()
{
	m_savedHGRLC = wglGetCurrentContext();
	m_savedHDC = wglGetCurrentDC();
	BOOL bResult = wglMakeCurrent( m_CDC->GetSafeHdc(), m_hOverlayGLRC);
	if ( bResult != TRUE )
		OutputDebugString("BADNESS!\n");

	
}

void CGLView::EndGLOverlayCalls()
{
	wglMakeCurrent( m_savedHDC, m_savedHGRLC );
}

void CGLView::SwapOverlayBuffers()
{
	wglSwapLayerBuffers( m_CDC->GetSafeHdc(), WGL_SWAP_OVERLAY1 );
}


// CGLView

IMPLEMENT_DYNCREATE(CGLView, CView)

CGLView::
CGLView()
{
}



CGLView
::~CGLView()
{
}

BEGIN_MESSAGE_MAP(CGLView, CView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
END_MESSAGE_MAP()


// CGLView drawing

void 
CGLView::
OnDraw(CDC* pDC)
{
	// want to call the client DrawGL callback, so we have to make
	// our context current 
	wglMakeCurrent(m_CDC->GetSafeHdc(), m_hGLRC);
	OnDrawGL();

	// swap buffers and free context
	SwapBuffers(m_CDC->GetSafeHdc());
	wglMakeCurrent(NULL, NULL);
}


// new registered window class in UNICODE
#define GLVIEW_UNICODE_CLASSNAME _T("GLVIEW_WINDOW_CLASS")

BOOL 
CGLView::
PreCreateWindow(CREATESTRUCT& cs)
{
   // OpenGL needs these window styles?
   cs.style |= (WS_CLIPCHILDREN | WS_CLIPSIBLINGS);          
 
   // call base class PreCreateWindow to get the cs.lpszClass filled in with the MFC default class name
   if( ! CView::PreCreateWindow(cs) )
     return 0;

   // Register the window class if it has not already been registered.
  
   WNDCLASS wndclass;
   HINSTANCE hInst = AfxGetInstanceHandle();

   // check if our class has been already registered (typical in MDI environment)
   if(!(::GetClassInfo(hInst, GLVIEW_UNICODE_CLASSNAME, &wndclass)))
   {
        // get default MFC class settings
		if(::GetClassInfo(hInst, cs.lpszClass, &wndclass))
		{
			// set window class name
			wndclass.lpszClassName = GLVIEW_UNICODE_CLASSNAME;

			// these styles are set for GL to work in MDI
			//   OWNDC: each window has it's own Device Context 
			//   HREDRAW/VREDRAW: redraw entire window if height/width changes
			wndclass.style |= (CS_OWNDC | CS_HREDRAW | CS_VREDRAW);

			// no background brush, because GL is going to draw our background
			wndclass.hbrBackground = NULL;

			// try to register class (else throw exception)
			if (!AfxRegisterClass(&wndclass)) 
				AfxThrowResourceException();
		}else
			AfxThrowResourceException();   // default MFC class not registered
	}

	// set our class name in CREATESTRUCT - this will work now, because we
	// have registered our window class
    cs.lpszClass = GLVIEW_UNICODE_CLASSNAME;

	return 1; 
}


// CGLView message handlers


int 
CGLView::
OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	// create new Client Device Context - represents client window area
	m_CDC = new CClientDC(this);
	
	// Set the pixel format for the window's DC. The pixel format determines
	// the drawing mode (RGBA, indexed, etc), depth buffer bits, and so on.
	// This has to happen *before* we create the GLRC
	if( SetPixelFormat() == FALSE )
		return 0;

	// Create OpenGL Rendering Context that will work with the window's pixel format
	m_hGLRC = wglCreateContext(m_CDC->GetSafeHdc());

	// now we want to possibly run some OpenGL calls, so we
	// make our GLRC current and set it to draw to this window's DC.
	// We call the CreateGL callback, and then free the DC by
	// setting the current GLRC to NULL
	wglMakeCurrent(m_CDC->GetSafeHdc(), m_hGLRC);
	OnCreateGL();
	wglMakeCurrent(NULL, NULL);

	// create overlay context, but do not make it current...
	m_hOverlayGLRC = wglCreateLayerContext(m_CDC->GetSafeHdc(), 1);

	LAYERPLANEDESCRIPTOR lpd;
	BOOL bResult = ::wglDescribeLayerPlane( m_CDC->GetSafeHdc(), m_nPixelFormat, 1, sizeof(lpd), &lpd );
	if ( bResult != TRUE )
		OutputDebugString("Error describing layer plane!\n");

	char buf[256];
	bool bOpenGL = (lpd.dwFlags & LPD_SUPPORT_OPENGL) != 0;
	bool bDbl = (lpd.dwFlags & LPD_DOUBLEBUFFER) != 0;
	bool bRGB = lpd.iPixelType == LPD_TYPE_RGBA;
	int nColorBits = lpd.cColorBits;
	int nAlphaBits = lpd.cAlphaBits;
	sprintf(buf, "GL: %d  DB: %d  RGB: %d  CBits: %d  ABits: %d\n",
		(int)bOpenGL, (int)bDbl, (int)bRGB, (int)nColorBits, (int)nAlphaBits );
	OutputDebugString(buf);

	return 0;
}


void 
CGLView::
OnDestroy()
{
	// delete the GL Rendering Context
	wglMakeCurrent(m_CDC->GetSafeHdc(), m_hGLRC);
	if( m_hGLRC != NULL)
		wglDeleteContext(m_hGLRC);

	// delete the overlay rendering context
	wglMakeCurrent(m_CDC->GetSafeHdc(), m_hOverlayGLRC);
	if ( m_hOverlayGLRC != NULL )
		wglDeleteContext( m_hOverlayGLRC );

	// destroy our client DeviceContext
	if(m_CDC != NULL)
		delete m_CDC;

	// destroy base class
	CView::OnDestroy();
}


BOOL 
CGLView::
OnEraseBkgnd(CDC* pDC)
{
	// skip default windows background clearing to avoid flicker (use GLClear)
	return TRUE;
}


void 
CGLView::
OnSize(UINT nType, int cx, int cy)
{
	CView::OnSize(nType, cx, cy);
	
	// if window size is 0, ignore
	if( cx <= 0 || cy <= 0)
		return;

	// call SizeGL callback (need to set GL rendering context first, and free it afterwards)
	wglMakeCurrent(m_CDC->GetSafeHdc(), m_hGLRC);
	OnSizeGL(cx,cy);
	wglMakeCurrent(NULL,NULL);

	// redraw
	Invalidate(TRUE);
}



// pixel format bits


BOOL CGLView::SetPixelFormat()
{

	// define a default desired video mode (pixel format)
	static PIXELFORMATDESCRIPTOR pfd = 
	{
        sizeof(PIXELFORMATDESCRIPTOR),	// size of this pfd
        1,								// version number
        PFD_DRAW_TO_WINDOW |			// support window
        PFD_SUPPORT_OPENGL |			// support OpenGL
        PFD_DOUBLEBUFFER ,				// double buffered
        PFD_TYPE_RGBA,                  // RGBA type
        32,                             // 32-bit color depth
        0, 0, 0, 0, 0, 0,               // color bits ignored
        8,                              // 8-bit alpha buffer
        0,                              // shift bit ignored
        0,                              // no accumulation buffer
        0, 0, 0, 0,                     // accum bits ignored
        16,                             // 16-bit z-buffer
        0,                              // no stencil buffer
        0,                              // no auxiliary buffer
        PFD_MAIN_PLANE,                 // main layer
        1,                              // specify # of overlay planes
        0, 0, 0                         // layer masks ignored
    };


    // try to find a compatible PixelFormat
    ASSERT(m_CDC != NULL);
	if ( (m_nPixelFormat = ChoosePixelFormat(m_CDC->GetSafeHdc(), &pfd)) == 0 ){
        AfxMessageBox("ChoosePixelFormat failed");
        return FALSE;
    }

    // try to switch to selected PixelFormat    
	if (::SetPixelFormat(m_CDC->GetSafeHdc(), m_nPixelFormat, &pfd) == FALSE){

    	// failed, try a default?
		MessageBox("SetPixelFormat failed - Requested mode unavailable, Trying Default");
        m_nPixelFormat = 1;	
		if (DescribePixelFormat(m_CDC->GetSafeHdc(), m_nPixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd)==0){
			AfxMessageBox("SetPixelFormat failed (no OpenGL compatible video mode)");
			m_nPixelFormat = -1;
			return FALSE;
		}
    }

	DescribePixelFormat(m_CDC->GetSafeHdc(), m_nPixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
	char buf[256];
	sprintf(buf,"%d layer planes\n", pfd.bReserved);
	OutputDebugString(buf);


    return TRUE;
}



BOOL CGLView::SetOverlayPixelFormat()
{
	return TRUE;
}