// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#pragma once


#include <gl/gl.h>
#include <gl/glu.h>

// CGLUIView view

// You shouldn't need to touch anything here. The functions you want to
// overload are at the bottom, starting with OnCreateGL()

class CGLView : public CView
{
	DECLARE_DYNCREATE(CGLView)

protected:
	CGLView();           // protected constructor used by dynamic creation
	virtual ~CGLView();

	// These are private because you should not be overriding them. You should
	// use OnDrawGL()
private:
	virtual void OnDraw(CDC* pDC);      // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);


protected:
	DECLARE_MESSAGE_MAP()

	// These are protected because you should not be overriding them. You should
	// use OnCreateGL() and OnSizeGL()
protected:
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnDestroy();
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnSize(UINT nType, int cx, int cy);


protected:
	CDC * m_CDC;       // pointer to Windows Device Context for this View (?)
	HGLRC m_hGLRC;	   // Handle to OpenGL/WGL Rendering Context for this View
	HGLRC m_hOverlayGLRC;	// Handle to WGL rendering context for overlay plane

	virtual BOOL SetPixelFormat();  // can override this to set custom PixelFormat
	virtual BOOL SetOverlayPixelFormat();  // can override this to set custom PixelFormat

	HDC m_savedHDC;
	HGLRC m_savedHGRLC;

	int m_nPixelFormat;

public:

	void BeginGLCalls();    // call this before you run GL commands outside the callbacks
	void EndGLCalls();      // call this afterwards. This is important.

	void BeginGLOverlayCalls();    // call this before you run GL commands outside the callbacks
	void EndGLOverlayCalls();      // call this afterwards. This is important.
	void SwapOverlayBuffers();


	// override these to do your own GL bits
	virtual void OnCreateGL();
	virtual void OnSizeGL(int cx, int cy);
	virtual void OnDrawGL();
};


