// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#pragma once

#include <set>

#include "InputEventStream.h"
#include "ViewController.h"
#include "HUDButton.h"
#include "HUDToolbar.h"
#include "HUDPopUpToolbar.h"
#include <GestureEngine.h>
#include "Timer.h"

#include "rmsdebug.h"


// CHUDManager

class CHUDManager : public CWnd,
		public rmsui::IInputEventListener,
		public rmsui::IGestureEventListener,
		public IViewListener,
		public rmsui::ITimeEventListener,
		public rmsui::IHUDPopUpToolbarEventHandler
{
	DECLARE_DYNAMIC(CHUDManager)

public:
	CHUDManager();
	virtual ~CHUDManager();


/*
 * HUD item handling
 */
	void AddHUDButton( rmsui::HUDButton * pButton );
	void AddHUDToolbar( rmsui::HUDToolbar * pToolbar );

/*
 * render functions
 */
	void SetViewController( CViewController * pView );

	void Render();

/*
 * IInputEventListener events
 */
	virtual EventState ProcessInputEvent( const rmsui::InputEventData & event );

/* 
 * IGestureEventListener events
 */
	virtual bool OnLineStroke( rmsui::GestureLineStroke * pLineStroke );
	virtual bool OnVStroke( rmsui::GestureVStroke * pVStroke );
	virtual bool OnPointStroke( rmsui::GesturePointStroke * pPointStroke );

/*
 * IViewListener events
 */
	virtual void OnViewChanged();

/*
 * ITimeEventListener events
 */
	//! returns true if anything changed (ie if we need a re-draw)
	virtual bool OnTimeEvent( const rmsui::TimeEvent & event );

/*
 * IHUDPopUpToolbarEventHandler events
 */
	virtual void OnHUDPopUpToolbarExpand();
	virtual void OnHUDPopUpToolbarContract();


protected:

/*
 * camera
 */ 
	CViewController * m_pView;
	float m_fToHUDSpaceParams[4];
	void ToHUDParams( Wml::Vector2f & vPoint );

	rmsui::HUDButtonEventHandler * m_pViewDirectionHandlers[6];
	rmsui::HUDButton * m_pViewDirectionButtons[6];
	rmsui::HUDPopUpToolbar * m_pViewDirectionPopUp;

	rmsui::HUDButtonEventHandler * m_pZoomButtonHandler;
	rmsui::HUDButtonEventHandler * m_pPanButtonHandler;
	rmsui::HUDButtonEventHandler * m_pOrbitButtonHandler;

	rmsui::HUDEventHandler * m_pViewDirectionPopUpCloser;

	rmsui::HUDButton * m_pZoomButton;
	rmsui::HUDButton * m_pPanButton;
	rmsui::HUDButton * m_pOrbitButton;

	rmsui::HUDToolbar * m_pViewToolbar;


	std::set< rmsui::HUDItem * > m_vHUDItems;

	rmsui::HUDItem * m_pCaptureItem;

	// generic toolbar move handler
	rmsui::IHUDToolbarEventHandler * m_pToolbarMoveHandler;

	// stroke processing functions
	bool CheckLineStroke( const rmsui::GestureLineStroke * pLineStroke );
	bool CheckVStroke( const rmsui::GestureVStroke * pVStroke );

	// docking support
	void UpdateDockedItems();

protected:
	DECLARE_MESSAGE_MAP()
};


