// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


// LightingControlDialog.cpp : implementation file
//

#include "stdafx.h"
#include "BlobShop.h"
#include "LightingControlDialog.h"
#include ".\lightingcontroldialog.h"
#include "SetFloatValueDialog.h"


// CLightingControlDialog dialog

IMPLEMENT_DYNAMIC(CLightingControlDialog, CDialog)
CLightingControlDialog::CLightingControlDialog(CWnd* pParent /*=NULL*/, CLightingControlDialogListener * pListener)
	: CDialog(CLightingControlDialog::IDD, pParent)
	, m_nLightingMode(0)
	, m_nShowCoordAxes(0)
	, m_nMeshColorMode(0)
	, m_fAmbientValue(0)
{
	Create( IDD_LIGHTING_CONTROL_DIALOG, pParent );

	m_pListener = pListener;
	ASSERT(pListener);
}

CLightingControlDialog::~CLightingControlDialog()
{
}

void CLightingControlDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);

	DDX_Radio( pDX, IDC_LIGHTCONTROL_DIFFUSE_CHECK, m_nLightingMode );

	DDX_Check( pDX, IDC_LC_SHOW_AXES, m_nShowCoordAxes );

	DDX_Radio( pDX, IDC_LC_MESHCOLOR_CONSTANT, m_nMeshColorMode );

	DDX_Radio( pDX, IDC_LC_SHADEMODE_SMOOTH, m_nShadingMode );
}


BEGIN_MESSAGE_MAP(CLightingControlDialog, CDialog)
	ON_BN_CLICKED(IDC_LIGHTCONTROL_APPLY_BUTTON, OnBnClickedLightcontrolApplyButton)
	ON_WM_SHOWWINDOW()
	ON_BN_CLICKED(IDC_LC_BACKGROUNDCOLOR, OnBnClickedLcBackgroundcolor)
	ON_BN_CLICKED(IDC_LC_MESHCOLOR, OnBnClickedLcMeshcolor)
	ON_BN_CLICKED(IDC_LC_AMBIENTVALUE, OnBnClickedLcAmbientvalue)
END_MESSAGE_MAP()


// CLightingControlDialog message handlers

void CLightingControlDialog::OnBnClickedLightcontrolApplyButton()
{
	m_pListener->OnLightingDialogApplyButton();
}

BOOL CLightingControlDialog::PreCreateWindow(CREATESTRUCT& cs)
{
	return CDialog::PreCreateWindow(cs);
}

void CLightingControlDialog::OnShowWindow(BOOL bShow, UINT nStatus)
{
	if (bShow != FALSE) {
		SetWindowPos( &wndTopMost, 0,0,0,0, SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE );
	}

	CDialog::OnShowWindow(bShow, nStatus);
}

void CLightingControlDialog::OnBnClickedLcBackgroundcolor()
{
	CColorDialog dlg(m_bgColor, 0, this);
	if (dlg.DoModal() == IDOK) {
		m_bgColor = dlg.GetColor();
	}
}

void CLightingControlDialog::OnBnClickedLcMeshcolor()
{
	CColorDialog dlg(m_meshColor, 0, this);
	if (dlg.DoModal() == IDOK) {
		m_meshColor = dlg.GetColor();
	}
}

void CLightingControlDialog::OnBnClickedLcAmbientvalue()
{
	CSetFloatValueDialog dlg(0, 1, m_fAmbientValue, "Ambient Light", false, this);
	if (  dlg.DoModal() == IDOK ) {
		m_fAmbientValue = dlg.GetNewValue();
	}
}
