// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


// Standard3DView.h : interface of the CStandard3DView class
//


#pragma once

#include "SketchFrameworkDoc.h"

#include <WmlVector3.h>

#include <ExtendedWmlCamera.h>
#include <WmlExtSmoothCameraInterpolator.h>

#include <ToolManager.h>
#include <GestureEngine.h>
#include <Image.h>

#include "ChangeListener.h"

#include "GLView.h"
#include "SceneManager.h"
#include "SketchManager.h"
#include "RenderingManager.h"
#include "SurfaceManager.h"
#include "HUDManager.h"
#include "AnimationManager.h"
#include "ShadingOverlay.h"

#include "ViewController.h"

#include "3DViewChildWindowSet.h"

#include "InputEventStream.h"

#include "BackgroundProcessManager.h"
#include "Timer.h"

// temporary ?
#include "TriangleMeshRenderer.h"
#include <Geometry.h>

class CStandard3DView : public CGLView, public IChangeListener, public IViewListener
{
protected: // create from serialization only
	CStandard3DView();
	DECLARE_DYNCREATE(CStandard3DView)

// Attributes
public:
	CSketchFrameworkDoc* GetDocument() const;

	CSceneManager & GetSceneManager()
	{ return *m_pSceneManager; }

	/*
	 * IChangeListener interface
	 */
	virtual void OnSceneChanged();
	virtual void OnSketchChanged();
	virtual void OnTreeSelectionChanged( const std::vector<rmsui::SceneObject *> & newSelection );
	virtual void OnRenderParametersChanged();

	/*
	 * IViewListener interface
	 */
	virtual void OnViewChanged();

// Operations
public:

// Overrides
public:
	virtual void OnCreateGL();
	virtual void OnSizeGL(int cx, int cy);
	virtual void OnDrawGL();

protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);

// Implementation
public:
	virtual ~CStandard3DView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	DECLARE_MESSAGE_MAP()



// camera functions
protected:
	
/*
 * background processing
 */
	CBackgroundProcessManager * m_pProcessManagerThread;


/*
 * rendering / UI stuff
 */
	CSurfaceManager * m_pSurfaceManager;

	CRenderingManager * m_pRenderManager;
	CHUDManager * m_pHUDManager;
	bool m_bDrawHUD;
	bool m_bCaptureFrames;

	rmsui::TimeEventGenerator m_timeGen;
	unsigned int m_nTimeGenTimer;


	void CaptureFrame(rmsui::Image & image);


/*
 * sketch stuff
 */
	CSketchManager * m_pSketchManager;

	ShadingOverlay m_shadingOverlay;
	bool m_bShadingMode;


/*
 * animation stuff
 */
	CAnimationManager * m_pAnimationManager;
	bool m_bAnimating;
	bool m_bStepOneFrameOnly;
	float m_fCurTime;


	rmsui::Tool3DManager * m_pToolManager;

	rmsui::GestureEngine * m_pGestureEngine;

	CSceneManager * m_pSceneManager;

	C3DViewChildWindowSet * m_pChildWindows;


/*
 * mouse handling
 */
	CPoint m_ptLastMousePosition;
	bool m_bLeftMouseDown;
	bool m_bMiddleMouseDown;
	bool m_bRightMouseDown;
	bool m_bAltKeyDown;

	void UpdateMousePosition( const CPoint & point )
		{ m_ptLastMousePosition = point; }
	void UpdateMouseDown( UINT whichButton, bool bState );

/*
 * camera handling
 */
	Wml::ExtendedWmlCamera * m_pCamera;
	CViewController * m_pViewController;

	void ZoomView( const CPoint & newPoint );
	void PanView( const CPoint & newPoint );
	void OrbitView( const CPoint & newPoint );

	//! smooth interpolator for camera transitions
	Wml::SmoothCameraInterpolator m_cameraInterpolator;

	//! timer ID for smooth camera transition WM_TIMER events
	unsigned int m_nSmoothCameraTimer;

	void SmoothCameraTransition( const Wml::Vector3f & newTarget, const Wml::Vector3f & newEye );
	void SmoothCameraTransition_TimerEvent();



/*
 * development build stuff
 */
	bool m_bMenusUpdated;
	void ConfigureDevelBuildFeatures();

/*
 * windows event handlers
 */


public:

/*
 * mouse events
 */
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnMButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnMButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);

/*
 * keyboard events
 */
	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnSysKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnSysKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKillFocus(CWnd* pNewWnd);
	afx_msg void OnViewFieldimage();
	afx_msg void OnUpdateViewFieldimage(CCmdUI *pCmdUI);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnSetFocus(CWnd* pOldWnd);
	afx_msg void OnViewEnablesketching();
	afx_msg void OnUpdateViewEnablesketching(CCmdUI *pCmdUI);
	virtual BOOL OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo);
	afx_msg void OnViewCachingcontrol();
	afx_msg void OnStatisticsFieldvalueapproximationerror();
	afx_msg void OnStatisticsCachememoryusage();
	afx_msg void OnStatisticsCacheevaluation();
	afx_msg void OnFileSave();
	afx_msg void OnEditRefinemesh();
	afx_msg void OnEditIncreaseMeshRes();
	afx_msg void OnEditDecreaseMeshRes();
	afx_msg void OnEditCycletool();
	afx_msg void OnViewPerspectivecamera();
	afx_msg void OnUpdateViewPerspectivecamera(CCmdUI *pCmdUI);
	afx_msg void OnEditClearcutplane();
	afx_msg void OnEditTogglecaching();
	afx_msg void OnUpdateEditTogglecaching(CCmdUI *pCmdUI);
	afx_msg void OnViewdirectionsLeft();
	afx_msg void OnViewdirectionsRight();
	afx_msg void OnViewdirectionsTop();
	afx_msg void OnViewdirectionsBottom();
	afx_msg void OnViewdirectionsFront();
	afx_msg void OnViewdirectionsBack();
	afx_msg void OnWindowRaytrace();
	afx_msg void OnExportmeshObjformat();
	afx_msg void OnFileOpen();
	afx_msg void OnExportmeshMeshliteformat();
	afx_msg void OnEditQuickrefine();
	afx_msg void OnFileExportselection();
	afx_msg void OnUpdateFileExportselection(CCmdUI *pCmdUI);
	afx_msg void OnAnimationPlay();
	afx_msg void OnViewShowhud();
	afx_msg void OnUpdateViewShowhud(CCmdUI *pCmdUI);
	afx_msg void OnAnimationStop();
	afx_msg void OnAnimationRewind();
	afx_msg void OnAnimationSaveframes();
	afx_msg void OnUpdateAnimationSaveframes(CCmdUI *pCmdUI);
	afx_msg void OnAnimationPlayoneframe();
	afx_msg void OnViewdirectionsThreequarter();
};

#ifndef _DEBUG  // debug version in Standard3DView.cpp
inline CSketchFrameworkDoc* CStandard3DView::GetDocument() const
   { return reinterpret_cast<CSketchFrameworkDoc*>(m_pDocument); }
#endif

