// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"
#include "TriangleMeshRenderer.h"

#include <gl/GL.h>
#include <gl/GLU.h>

#include "rmsdebug.h"


void TriangleMeshRenderer::DrawAsZFill()
{
	float * vertices = m_pMesh->GetVertexPointer();
	std::vector<unsigned int> & triangles = m_pMesh->GetTriangles();

	glBegin(GL_TRIANGLES);
	unsigned int nTriCount = (unsigned int)triangles.size();
	for (unsigned int i = 0; i < nTriCount; i += 3) {
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i], 0 )] );		
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+1], 0 )] );		
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+2], 0 )] );	
	}
	glEnd();
}


void TriangleMeshRenderer::DrawImmediateVtxNormal() const
{
	// draw mesh
	float * vertices = m_pMesh->GetVertexPointer();
	float * normals = m_pMesh->GetNormalPointer();
	std::vector<unsigned int> & triangles = m_pMesh->GetTriangles();

	glBegin(GL_TRIANGLES);
	unsigned int nTriCount = (unsigned int)triangles.size();
	for (unsigned int i = 0; i < nTriCount; i += 3) {
		glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i], 0 )] );
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i], 0 )] );		
		glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i+1], 0 )] );
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+1], 0 )] );		
		glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i+2], 0 )] );
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+2], 0 )] );
	}
	glEnd();
}

void TriangleMeshRenderer::DrawImmediateVtxNormalAndColor() const
{
	// draw mesh
	float * vertices = m_pMesh->GetVertexPointer();
	float * normals = m_pMesh->GetNormalPointer();
	float * colors = m_pMesh->GetColorPointer();
	std::vector<unsigned int> & triangles = m_pMesh->GetTriangles();

	glBegin(GL_TRIANGLES);
	unsigned int nTriCount = (unsigned int)triangles.size();
	for (unsigned int i = 0; i < nTriCount; i += 3) {
		glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i], 0 )] );
		glColor3fv( &colors[TM_VERTEX_INDEX( triangles[i], 0)] );
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i], 0 )] );		

		glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i+1], 0 )] );
		glColor3fv( &colors[TM_VERTEX_INDEX( triangles[i+1], 0)] );
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+1], 0 )] );		

		glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i+2], 0 )] );
		glColor3fv( &colors[TM_VERTEX_INDEX( triangles[i+2], 0)] );
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+2], 0 )] );		
	}
	glEnd();
}

void TriangleMeshRenderer::DrawImmediateVtxNormalAndTex(bool bPerVtxTexCoords) const
{
	// draw mesh
	float * vertices = m_pMesh->GetVertexPointer();
	float * normals = m_pMesh->GetNormalPointer();
	const std::vector<unsigned int> & triangles = m_pMesh->GetTriangles();
	unsigned int nTriCount = (unsigned int)triangles.size();

	glBegin(GL_TRIANGLES);

	if ( !bPerVtxTexCoords && m_pMesh->HasTriangleTextureCoords() ) {
		float * pTriTextures = m_pMesh->GetTriTexCoordPointer();
		for (unsigned int i = 0; i < nTriCount; i += 3) {
			glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i], 0 )] );
			glTexCoord2fv( &pTriTextures[TM_TEXTURE_INDEX( i, 0)] );
			glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i], 0 )] );		

			glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i+1], 0 )] );
			glTexCoord2fv( &pTriTextures[TM_TEXTURE_INDEX( i, 2)] );
			glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+1], 0 )] );		

			glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i+2], 0 )] );
			glTexCoord2fv( &pTriTextures[TM_TEXTURE_INDEX( i, 4)] );
			glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+2], 0 )] );		
		}

	} else {
		float * textures = m_pMesh->GetTextureCoordsPointer();
		for (unsigned int i = 0; i < nTriCount; i += 3) {
			glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i], 0 )] );
			glTexCoord2fv( &textures[TM_TEXTURE_INDEX( triangles[i], 0)] );
			glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i], 0 )] );		

			glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i+1], 0 )] );
			glTexCoord2fv( &textures[TM_TEXTURE_INDEX( triangles[i+1], 0)] );
			glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+1], 0 )] );		

			glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i+2], 0 )] );
			glTexCoord2fv( &textures[TM_TEXTURE_INDEX( triangles[i+2], 0)] );
			glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+2], 0 )] );		
		}
	}

	glEnd();

}




void TriangleMeshRenderer::DrawImmediateTriNormal() const
{
	// draw mesh
	float * vertices = m_pMesh->GetVertexPointer();
	float * normals = m_pMesh->GetNormalPointer();
	std::vector<unsigned int> & triangles = m_pMesh->GetTriangles();

	glBegin(GL_TRIANGLES);
	unsigned int nTriCount = (unsigned int)triangles.size();
	for (unsigned int i = 0; i < nTriCount; i += 3) {

		Wml::Vector3f v1( &vertices[TM_VERTEX_INDEX( triangles[i], 0 )] );
		Wml::Vector3f v2( &vertices[TM_VERTEX_INDEX( triangles[i+1], 0 )] );
		Wml::Vector3f v3( &vertices[TM_VERTEX_INDEX( triangles[i+2], 0 )] );
		Wml::Vector3f vEdge1(v2-v1);
		Wml::Vector3f vNormal( vEdge1.Cross(v3-v1) );
		vNormal.Normalize();
		glNormal3fv( vNormal );

		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i], 0 )] );		
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+1], 0 )] );		
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+2], 0 )] );
	}
	glEnd();
}

void TriangleMeshRenderer::DrawImmediateTriNormalAndColor() const
{
	// draw mesh
	float * vertices = m_pMesh->GetVertexPointer();
	float * normals = m_pMesh->GetNormalPointer();
	float * colors = m_pMesh->GetColorPointer();
	std::vector<unsigned int> & triangles = m_pMesh->GetTriangles();

	glBegin(GL_TRIANGLES);
	unsigned int nTriCount = (unsigned int)triangles.size();
	for (unsigned int i = 0; i < nTriCount; i += 3) {

		Wml::Vector3f v1( &vertices[TM_VERTEX_INDEX( triangles[i], 0 )] );
		Wml::Vector3f v2( &vertices[TM_VERTEX_INDEX( triangles[i+1], 0 )] );
		Wml::Vector3f v3( &vertices[TM_VERTEX_INDEX( triangles[i+2], 0 )] );
		Wml::Vector3f vEdge1(v2-v1);
		Wml::Vector3f vNormal( vEdge1.Cross(v3-v1) );
		vNormal.Normalize();
		glNormal3fv( vNormal );

		glColor3fv( &colors[TM_VERTEX_INDEX( triangles[i], 0)] );
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i], 0 )] );		

		glColor3fv( &colors[TM_VERTEX_INDEX( triangles[i+1], 0)] );
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+1], 0 )] );		

		glColor3fv( &colors[TM_VERTEX_INDEX( triangles[i+2], 0)] );
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+2], 0 )] );		
	}
	glEnd();
}

void TriangleMeshRenderer::DrawImmediateTriNormalAndTex(bool bPerVtxTexCoords) const
{
	// draw mesh
	float * vertices = m_pMesh->GetVertexPointer();
	float * normals = m_pMesh->GetNormalPointer();
	float * textures = m_pMesh->GetTextureCoordsPointer();
	std::vector<unsigned int> & triangles = m_pMesh->GetTriangles();

	glBegin(GL_TRIANGLES);
	unsigned int nTriCount = (unsigned int)triangles.size();
	for (unsigned int i = 0; i < nTriCount; i += 3) {

		Wml::Vector3f v1( &vertices[TM_VERTEX_INDEX( triangles[i], 0 )] );
		Wml::Vector3f v2( &vertices[TM_VERTEX_INDEX( triangles[i+1], 0 )] );
		Wml::Vector3f v3( &vertices[TM_VERTEX_INDEX( triangles[i+2], 0 )] );
		Wml::Vector3f vEdge1(v2-v1);
		Wml::Vector3f vNormal( vEdge1.Cross(v3-v1) );
		vNormal.Normalize();
		glNormal3fv( vNormal );

		glTexCoord2fv( &textures[TM_TEXTURE_INDEX( triangles[i], 0)] );
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i], 0 )] );		

		glTexCoord2fv( &textures[TM_TEXTURE_INDEX( triangles[i+1], 0)] );
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+1], 0 )] );		

		glTexCoord2fv( &textures[TM_TEXTURE_INDEX( triangles[i+2], 0)] );
		glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+2], 0 )] );		
	}
	glEnd();
}



void TriangleMeshRenderer::DrawImmediate( bool bUseTriNormals, bool bUseMeshColor, bool bOutputTexCoords, bool bPerVtxTexCoords ) const
{
	if ( bUseTriNormals ) {
		if ( bUseMeshColor )
			DrawImmediateTriNormalAndColor();
		else if ( bOutputTexCoords )
			DrawImmediateTriNormalAndTex(bPerVtxTexCoords);
		else
			DrawImmediateTriNormal();

	} else {
		if ( bUseMeshColor )
			DrawImmediateVtxNormalAndColor();
		else if ( bOutputTexCoords )
			DrawImmediateVtxNormalAndTex(bPerVtxTexCoords);
		else
			DrawImmediateVtxNormal();
	}

}



void TriangleMeshRenderer::DrawImmediate(bool bUseTriNormals) const
{
	// draw mesh
	float * vertices = m_pMesh->GetVertexPointer();
	float * normals = m_pMesh->GetNormalPointer();
	std::vector<unsigned int> & triangles = m_pMesh->GetTriangles();

	glBegin(GL_TRIANGLES);
	unsigned int nTriCount = (unsigned int)triangles.size();
	if (bUseTriNormals) {
		for (unsigned int i = 0; i < nTriCount; i += 3) {
			Wml::Vector3f v1( &vertices[TM_VERTEX_INDEX( triangles[i], 0 )] );
			Wml::Vector3f v2( &vertices[TM_VERTEX_INDEX( triangles[i+1], 0 )] );
			Wml::Vector3f v3( &vertices[TM_VERTEX_INDEX( triangles[i+2], 0 )] );
			Wml::Vector3f vEdge1(v2-v1);
			Wml::Vector3f vNormal( vEdge1.Cross(v3-v1) );
			vNormal.Normalize();
			glNormal3fv( vNormal );
			glVertex3fv( v1 );		
			glVertex3fv( v2 );		
			glVertex3fv( v3 );	
		}

	} else {

		for (unsigned int i = 0; i < nTriCount; i += 3) {
			glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i], 0 )] );
			glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i], 0 )] );		
			glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i+1], 0 )] );
			glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+1], 0 )] );		
			glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i+2], 0 )] );
			glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+2], 0 )] );
		}
	}

	glEnd();
}





void TriangleMeshRenderer::DrawImmediateWithColor(bool bUseTriNormals) const
{
	// draw mesh
	float * vertices = m_pMesh->GetVertexPointer();
	float * normals = m_pMesh->GetNormalPointer();
	float * colors = m_pMesh->GetColorPointer();
	std::vector<unsigned int> & triangles = m_pMesh->GetTriangles();

	glBegin(GL_TRIANGLES);
	unsigned int nTriCount = (unsigned int)triangles.size();

	if (bUseTriNormals) {
		for (unsigned int i = 0; i < nTriCount; i += 3) {
			Wml::Vector3f v1( &vertices[TM_VERTEX_INDEX( triangles[i], 0 )] );
			Wml::Vector3f v2( &vertices[TM_VERTEX_INDEX( triangles[i+1], 0 )] );
			Wml::Vector3f v3( &vertices[TM_VERTEX_INDEX( triangles[i+2], 0 )] );
			Wml::Vector3f vEdge1(v2-v1);
			Wml::Vector3f vNormal( vEdge1.Cross(v3-v1) );
			vNormal.Normalize();
			glNormal3fv( vNormal );
			glColor3fv( &colors[TM_VERTEX_INDEX( triangles[i], 0)] );
			glVertex3fv( v1 );		
			glColor3fv( &colors[TM_VERTEX_INDEX( triangles[i+1], 0)] );
			glVertex3fv( v2 );		
			glColor3fv( &colors[TM_VERTEX_INDEX( triangles[i+2], 0)] );
			glVertex3fv( v3 );	
		}

	} else {

		for (unsigned int i = 0; i < nTriCount; i += 3) {
			glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i], 0 )] );
			glColor3fv( &colors[TM_VERTEX_INDEX( triangles[i], 0)] );
			glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i], 0 )] );		
			glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i+1], 0 )] );
			glColor3fv( &colors[TM_VERTEX_INDEX( triangles[i+1], 0)] );
			glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+1], 0 )] );		
			glNormal3fv( &normals[TM_VERTEX_INDEX( triangles[i+2], 0 )] );
			glColor3fv( &colors[TM_VERTEX_INDEX( triangles[i+2], 0)] );
			glVertex3fv( &vertices[TM_VERTEX_INDEX( triangles[i+2], 0 )] );		
		}
	}

	glEnd();
}

void TriangleMeshRenderer::DrawVerticesAsPoints() const
{
	Wml::Vector3f vtx;
	glBegin(GL_POINTS);
	unsigned int nVtxCount = m_pMesh->GetNumVertices();
	for (unsigned int i = 0; i < nVtxCount; ++i) {
		m_pMesh->GetVertex(i, vtx);
		glVertex3fv(vtx);
	}
	glEnd();
}


void TriangleMeshRenderer::DrawVertexBuffer() const
{
	// not done yet
}

void TriangleMeshRenderer::DrawNormals( float fLength ) const
{
	std::vector<float> & vertices = m_pMesh->GetVertices();
	std::vector<float> & normals = m_pMesh->GetNormals();

	if (vertices.size() != normals.size())
		return;

	glBegin(GL_LINES);
	unsigned int nCount = (unsigned int)vertices.size();
	for (unsigned int i = 0; i < nCount; i += TM_VERTEX_STRIDE) {
		glVertex3fv( &vertices[i] );
		float v[3] = { vertices[i] + fLength * normals[i],
					   vertices[i+1] + fLength * normals[i+1],
					   vertices[i+2] + fLength * normals[i+2] };
		glVertex3fv( v );
	}
	glEnd();
}


void TriangleMeshRenderer::DrawCameraProjection( float fLength, const Wml::Vector3f & vCameraPos ) const
{
	Wml::Vector3f vVertex, vNormal;

	glBegin(GL_LINES);
	unsigned int nCount = m_pMesh->GetNumVertices();
	for (unsigned int i = 0; i < nCount; i += TM_VERTEX_STRIDE) {

		m_pMesh->GetVertex(i, vVertex);
		m_pMesh->GetNormal(i, vNormal);

		Wml::Vector3f vCamVec( vVertex );
		vCamVec -= vCameraPos;

		// project camera vec onto normal
		Wml::Vector3f vProjN( vNormal * vNormal.Dot(vCamVec) );

		// now get W vector 
		Wml::Vector3f vW = vCamVec - vProjN;
		vW.Normalize();	

		glVertex3fv( vVertex );
		vVertex += vW * fLength;
		glVertex3fv( vVertex );
	}
	glEnd();
}


void TriangleMeshRenderer::DrawCurvatureDirections( float fLength, bool bFirst, bool bSecond, rmsimplicit::ScalarField * pField ) const
{
	float fCurv1, fCurv2;
	Wml::Vector3f vVertex, vCurv1, vCurv2;

	glLineWidth(6.0f);
		
	glBegin(GL_LINES);
	unsigned int nCount = m_pMesh->GetNumVertices();
	for (unsigned int i = 0; i < nCount; i += TM_VERTEX_STRIDE) {

		m_pMesh->GetVertex(i, vVertex);

		pField->PrincipalCurvatures( vVertex.X(), vVertex.Y(), vVertex.Z(),
			fCurv1, fCurv2, vCurv1, vCurv2 );
		vCurv1.Normalize(); vCurv2.Normalize();

		if ( bFirst ) {
			glColor3f(1,0,0);
			glVertex3fv( vVertex );
			vCurv1 *= fLength;
			vCurv1 += vVertex;
			glColor3f(0,0,0);
			glVertex3fv( vCurv1 );
		}

		if ( bSecond ) {
			glColor3f(1,0,0);
			glVertex3fv( vVertex );
			vCurv2 *= fLength;
			vCurv2 += vVertex;
			glColor3f(1,1,1);
			glVertex3fv( vCurv2 );
		}
	}
	glEnd();
}
