// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#pragma once

#include <ExtendedWmlCamera.h>
#include <set>


class IViewListener
{
public:
	virtual void OnViewChanged() = 0;
};

class CViewController
{
public:
	CViewController(Wml::ExtendedWmlCamera * pCamera);
	~CViewController(void);

	Wml::ExtendedWmlCamera * GetCamera() { return m_pCamera; }

	const unsigned int * GetViewport() { return m_pCamera->GetViewport(); }

	void ForceRepaint();

	void UpdateCamera();
	void DollyZoom( float fDX, float fDY );
	void Pan( float fDX, float fDY );
	void Orbit( float fDX, float fDY );

	void ViewLeft();
	void ViewRight();
	void ViewTop();
	void ViewBottom();
	void ViewFront();
	void ViewBack();

	void ViewThreeQuarter();

	void RegisterListener( IViewListener * pListener ) { m_vListeners.insert(pListener); }
	void RemoveListener( IViewListener * pListener ) { m_vListeners.erase(pListener); }


/*
 * Serialization
 */
	virtual Wml::SerializerRecord * Serialize( Wml::Serializer * pSerializer );
	virtual bool Serialize( Wml::SerializerRecord * pRecord, Wml::Serializer * pSerializer );


protected:
	Wml::ExtendedWmlCamera * m_pCamera;

	std::set< IViewListener * > m_vListeners;

	void NotifyViewChanged();
};
