#include "StdAfx.h"
#include <WmlDistLin3Lin3.h>

#include "DynamicTool3D.h"
#include "rmsdebug.h"
#include <limits>

using namespace rmsui;

DynamicTool3D::DynamicTool3D()
	: ITool3D("DynamicTool3D")
{
	m_pCurConstraint = NULL;
}

void DynamicTool3D::SetTarget( ITool3DTarget * pTarget )
{
	ITool3D::SetTarget(pTarget);
}

void DynamicTool3D::TargetFrameChanged()
{	
	ITool3D::TargetFrameChanged();
}


void DynamicTool3D::Render( const ViewState & viewState ) 
{
	m_vPivots.Render(viewState);

	m_vConstraints.Render(viewState);
}


bool DynamicTool3D::Intersect( const Wml::Ray3f & ray, const ViewState & viewState,
							   Wml::Vector3f & ptPosition, float & fDepth )
{
	int nHitID = 0;
	Constraint * pHit = m_vConstraints.Intersect(ray, ptPosition, nHitID);
	if ( pHit ) {
		m_pCurConstraint = pHit;
		fDepth = (ptPosition - ray.Origin()).Length();
		return true;
	}
	return false;
}


Wml::Vector3f DynamicTool3D::Move( SelectedTargetInfo & targetInfo,
								   const ViewState & viewState,
								   bool bInitialize )
{
	if ( m_pCurConstraint == NULL )
		return Wml::Vector3f::ZERO;

	Wml::Frame3f vCurFrame( GetFrame() );

	if ( dynamic_cast<VectorConstraint *>(m_pCurConstraint) != NULL ) {
		if ( bInitialize )
			m_pCurConstraint->InitializeConstraint(targetInfo.eyeViewRay, vCurFrame);
		else {
			Wml::Vector3f vTranslate;
			bool bRelative;
			m_pCurConstraint->GetFrameTranslation( targetInfo.eyeViewRay, vCurFrame, vTranslate, bRelative );
			m_pCurConstraint->Target()->TranslateFrame( vTranslate, bRelative );
		}
	}

	if ( dynamic_cast<PlaneTranslateConstraint *>(m_pCurConstraint) != NULL ) {
		if ( bInitialize )
			m_pCurConstraint->InitializeConstraint(targetInfo.eyeViewRay, vCurFrame);
		else {
			Wml::Vector3f vTranslate;
			bool bRelative;
			m_pCurConstraint->GetFrameTranslation( targetInfo.eyeViewRay, vCurFrame, vTranslate, bRelative );
			m_pCurConstraint->Target()->TranslateFrame( vTranslate, bRelative );
		}
	}

	if ( dynamic_cast<AxisRotationConstraint *>(m_pCurConstraint) != NULL ) {
		if ( bInitialize )
			m_pCurConstraint->InitializeConstraint( targetInfo.eyeViewRay, vCurFrame );
		else {
			Wml::Frame3f vNewFrame;
			Wml::Matrix3f rotation;
			m_pCurConstraint->GetFrameRotation( targetInfo.eyeViewRay, vCurFrame, rotation );
			m_pCurConstraint->Target()->RotateFrame(rotation);
		}
	}

	if ( dynamic_cast<UniformScaleConstraint *>(m_pCurConstraint) != NULL ) {
		if ( bInitialize )
			m_pCurConstraint->InitializeConstraint( targetInfo.eyeViewRay, vCurFrame );
		else {
			float fScale;
			m_pCurConstraint->GetUniformScaleDelta(targetInfo.eyeViewRay, vCurFrame, fScale);

			// fScale is an absolute delta increment (allows scale to go negative), but we 
			// might as well convert it to a relative scaling factor r here...
			Wml::Vector3f vCurScale = m_pCurConstraint->Target()->SceneObj()->GetFrame().Scale();
			Wml::Vector3f vNewScale(vCurScale.X() + fScale, vCurScale.Y() + fScale, vCurScale.Z() + fScale);
			Wml::Vector3f vScale(vNewScale.X() / vCurScale.X(), vNewScale.Y() / vCurScale.Y(), vNewScale.Z() / vCurScale.Z() );
			m_pCurConstraint->Target()->Scale(vScale.X(), vScale.Y(), vScale.Z(), true, false);

		}
	}

	if ( dynamic_cast<AxisScaleConstraint *>(m_pCurConstraint) != NULL ) {
		if ( bInitialize )
			m_pCurConstraint->InitializeConstraint( targetInfo.eyeViewRay, vCurFrame );
		else {
			Wml::Vector3f vScale;
			m_pCurConstraint->GetScaleDelta(targetInfo.eyeViewRay, vCurFrame, vScale);

			// fScale is an absolute delta increment (allows scale to go negative), but we 
			// might as well convert it to a relative scaling factor r here...
			Wml::Vector3f vCurScale = m_pCurConstraint->Target()->SceneObj()->GetFrame().Scale();
			Wml::Vector3f vNewScale(vCurScale + vScale);
			vScale = Wml::Vector3f(vNewScale.X() / vCurScale.X(), vNewScale.Y() / vCurScale.Y(), vNewScale.Z() / vCurScale.Z() );
			m_pCurConstraint->Target()->Scale(vScale.X(), vScale.Y(), vScale.Z(), true, false);

		}
	}



	return Wml::Vector3f::ZERO;
}



void DynamicTool3D::ClearConstraints(bool bPinned)
{
	m_vConstraints.Clear(bPinned);
	m_pCurConstraint = NULL;
}

