// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"
#include "GenericPointSceneObject.h"

#include <gl/GL.h>
#include <gl/GLU.h>
#include <WmlDistVec3Lin3.h>

using namespace rmsui;

GenericPointSceneObject::GenericPointSceneObject( GenericPointSceneObjectTarget * pListener ) 
{ 
	m_pParent = NULL; 

	m_pListener = pListener;
}

//! try to pick object. Returns NULL on failure, otherwise returns picked SceneObject (may be a child!)
SceneObject * GenericPointSceneObject::Pick( const Wml::Ray3f & ray, float & fSqrDistance )
{
	Wml::Vector3f vPoint = m_pListener->GetPosition();

	float fRayParam;
	float fSqrDist = Wml::SqrDistance( vPoint, ray, &fRayParam );
	if (fRayParam > 0.0 && fSqrDist < 0.005 ) {
		fSqrDistance = fRayParam;
		return this;
	}
	return NULL;
}

//! render object
void GenericPointSceneObject::Render(bool bSelected)
{
	Wml::Vector3f vPoint = m_pListener->GetPosition();

	glBegin(GL_POINTS);
	glVertex3f( vPoint.X(), vPoint.Y(), vPoint.Z() );
	glEnd();
}


inline Wml::Frame GenericPointSceneObject::GetFrame()
{
	return Wml::Frame( m_pListener->GetPosition() );
}

void GenericPointSceneObject::Translate( const Wml::Vector3f & vTranslate, bool bRelative )
{
	m_pListener->Translate( vTranslate, bRelative );
}

//! test if target supports a tool
bool GenericPointSceneObject::SupportsTool( const std::string & sToolName )
{
	if ( sToolName == "Translation3D" )
		return true;
	return false;
}