// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef __RMSUI_GENERIC_POINT_SCENE_OBJECT_H__
#define __RMSUI_GENERIC_POINT_SCENE_OBJECT_H__

#include <vector>
#include "SceneObject.h"


namespace rmsui {


class GenericPointSceneObjectTarget
{
public:
	virtual ~GenericPointSceneObjectTarget() {}

	virtual Wml::Vector3f GetPosition() = 0;

	virtual void Translate( const Wml::Vector3f & vTranslate, bool bRelative = true ) = 0;
};


class GenericPointSceneObject : public SceneObject
{
public:
	GenericPointSceneObject( GenericPointSceneObjectTarget * pListener );
	virtual ~GenericPointSceneObject(void) {};

	//! try to pick object. Returns NULL on failure, otherwise returns picked SceneObject (may be a child!)
	virtual SceneObject * Pick( const Wml::Ray3f & ray, float & fRayParam );

	//! render object
	virtual void Render(bool bSelected = false);


	/*
	 * ITool3DTarget functions
	 */

	//! get the frame for this target
	virtual Wml::Frame GetFrame();

	//! translate the frame
	virtual void Translate( const Wml::Vector3f & vTranslate, bool bRelative = true );

	//! test if target supports a tool
	virtual bool SupportsTool( const std::string & sToolName );

protected:
	GenericPointSceneObjectTarget * m_pListener;
};






}  // end namespace rmsui


#endif  // __RMSUI_GENERIC_POINT_SCENE_OBJECT_H__