// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef _RMSUI_GESTURE_STROKES_H_
#define _RMSUI_GESTURE_STROKES_H_

#include <string>
#include <set>
#include <vector>

#include <WmlAxisAlignedBox2.h>
#include <WmlBox2.h>
#include <WmlVector2.h>

namespace rmsui
{

class GestureLineStroke;
class GesturePointStroke;
class GestureVStroke;
class GestureScribbleStroke;

class GestureStroke
{
public:
	GestureStroke();
	GestureStroke( const std::vector<Wml::Vector2f> & vVertices );
	virtual ~GestureStroke();

	void AppendVertex( const Wml::Vector2f & vVertex, bool bSkipDuplicates = true );

	inline unsigned int GetNumVertices();
	inline const std::vector<Wml::Vector2f> & GetVertices();
	inline Wml::Vector2f GetVertex( unsigned int nVertex );

	inline const Wml::Vector2f * GetLastVertex();

	Wml::Box2f GetBoundingBox();

	void DrawGL();

	bool IsMultipleLoopsStroke();
	bool IsLoopStroke();

	bool IsLineStroke();
	bool GetLineStroke( GestureLineStroke * pStroke );

	bool IsPointStroke();
	bool GetPointStroke( GesturePointStroke * pStroke );

	bool IsVStroke();
	bool GetVStroke( GestureVStroke * pStroke );

	bool IsScribbleStroke();
	bool GetScribbleStroke( GestureScribbleStroke * pScribbleStroke );

protected:
	std::vector< Wml::Vector2f > m_vVertices;

	bool IsLine( unsigned int nStartVertex, unsigned int nEndVertex );
};




class GestureLineStroke
{
public:
	GestureLineStroke();
	GestureLineStroke( const Wml::Vector2f & pt1, const Wml::Vector2f & pt2 );
	~GestureLineStroke();

	Wml::Vector2f & EndPoint1() { return m_pt1; }
	Wml::Vector2f & EndPoint2() { return m_pt2; }

	const Wml::Vector2f & EndPoint1() const { return m_pt1; }
	const Wml::Vector2f & EndPoint2() const { return m_pt2; }

protected:
	Wml::Vector2f m_pt1;
	Wml::Vector2f m_pt2;
};


class GesturePointStroke
{
public:
	GesturePointStroke();
	GesturePointStroke( const Wml::Vector2f & pt );
	~GesturePointStroke();

	Wml::Vector2f & Point() { return m_pt; }

	const Wml::Vector2f & Point() const { return m_pt; }

protected:
	Wml::Vector2f m_pt;
};


class GestureVStroke
{
public:
	GestureVStroke();
	GestureVStroke( const Wml::Vector2f & ptStart, const Wml::Vector2f & ptEnd );
	~GestureVStroke();

	Wml::Vector2f & Start() { return m_ptStart; }
	const Wml::Vector2f & Start() const { return m_ptStart; }

	Wml::Vector2f & End() { return m_ptEnd; }
	const Wml::Vector2f & End() const { return m_ptEnd; }

protected:
	Wml::Vector2f m_ptStart;
	Wml::Vector2f m_ptEnd;
};


class GestureScribbleStroke
{
public:
	GestureScribbleStroke();
	GestureScribbleStroke( const Wml::Box2f & BoundingBox );
	~GestureScribbleStroke();

	Wml::Box2f & Bounds() { return m_boundingBox; }
	const Wml::Box2f & Bounds() const { return m_boundingBox; }

protected:
	Wml::Box2f m_boundingBox;
};





/*
 * Stroke inlines
 */

inline unsigned int GestureStroke::GetNumVertices()
{
	return (unsigned int)m_vVertices.size();
}

inline const std::vector<Wml::Vector2f> & GestureStroke::GetVertices()
{
	return m_vVertices;
}

inline const Wml::Vector2f * GestureStroke::GetLastVertex()
{
	return (m_vVertices.size() > 0) ? &m_vVertices.back() : NULL;
}


inline Wml::Vector2f GestureStroke::GetVertex( unsigned int nVertex )
{
	return m_vVertices[nVertex];
}




}   // namespace rmsui



#endif // _RMSUI_Gesture_STROKES_H_