// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"
#include "HUDButton.h"

using namespace rmsui;

#include "Image.h"


HUDFileIcon::HUDFileIcon( const char * pFilename )
{
	m_filename = std::string(pFilename);

	Image img;
	if (!img.Load( m_filename.c_str() ))
		return;

	// generate texture buffer
	std::vector<unsigned char> vBuffer;
	Texture::MakeFromBuffer( img.Width(), img.Height(), 
		img.Channels(), &img.Buffer()[0], 
		Image::RGB_CHANNEL_MAP, true, vBuffer );

	// initialize texture
	m_texture.InitializeBuffer( img.Width(), img.Height(), vBuffer, GL_RGBA );
	m_texture.FlipY();
	m_texture.SetTextureInfo( GL_TEXTURE_2D, GL_LINEAR, GL_REPEAT );
	m_texture.TextureEnvMode() = GL_MODULATE;
}


void HUDFileIcon::Draw( const Wml::AxisAlignedBox2f & bounds )
{
	glColor4f(1,1,1,1);

	m_texture.Enable();

	glBegin(GL_QUADS);

	glTexCoord2f(0, 0);
	glVertex2f( bounds.GetXMin(), bounds.GetYMin() );
	glTexCoord2f(1, 0);
	glVertex2f( bounds.GetXMax(), bounds.GetYMin() );
	glTexCoord2f(1, 1);
	glVertex2f( bounds.GetXMax(), bounds.GetYMax() );
	glTexCoord2f(0, 1);
	glVertex2f( bounds.GetXMin(), bounds.GetYMax() );

	glEnd();

	m_texture.Disable();
}



HUDExternalImageIcon::HUDExternalImageIcon( Texture * pTexture )
{
	m_pTexture = pTexture;
}


void HUDExternalImageIcon::Draw( const Wml::AxisAlignedBox2f & bounds )
{
	glColor3f(1,1,1);

	m_pTexture->Enable();

	glBegin(GL_QUADS);

	glTexCoord2f(0, 0);
	glVertex2f( bounds.GetXMin(), bounds.GetYMin() );
	glTexCoord2f(1, 0);
	glVertex2f( bounds.GetXMax(), bounds.GetYMin() );
	glTexCoord2f(1, 1);
	glVertex2f( bounds.GetXMax(), bounds.GetYMax() );
	glTexCoord2f(0, 1);
	glVertex2f( bounds.GetXMin(), bounds.GetYMax() );

	glEnd();

	m_pTexture->Disable();
}


