// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef _RMSUI_HUD_POPUP_TOOLBAR_H_
#define _RMSUI_HUD_POPUP_TOOLBAR_H_

#include "HUDItem.h"
#include "HUDButton.h"
#include <vector>

namespace rmsui
{

class IHUDPopUpToolbarEventHandler : public HUDEventHandler
{
public:
	virtual ~IHUDPopUpToolbarEventHandler() {}

	virtual void OnHUDPopUpToolbarExpand() {}
	virtual void OnHUDPopUpToolbarContract() {}
};


class HUDPopUpToolbar : public HUDItem
{
public:

	enum Orientation {
		Horizontal, Vertical
	};

	HUDPopUpToolbar(const std::string & name, Orientation eOrientation = Horizontal );
	virtual ~HUDPopUpToolbar();

	/*
	 * overrides
	 */
	virtual HUDItem * BeginConsume( const Wml::Vector2f & vPoint );
	virtual HUDItem * Consume( const Wml::Vector2f & vNextPoint );
	virtual void EndConsume();

	virtual HUDItem * TestLineStroke( const GestureLineStroke & stroke );

	virtual void Render();

	virtual void Translate( float fX, float fY, bool bRelative );

	/*
	 * button control
	 */
	void AppendItem( HUDItem * pItem );
	void ClearItems();
	

	/*
	 * toolbar options
	 */
	Orientation GetOrientation() { return m_eOrientation; }
	void SetOrientation( Orientation eOrientation );

	float GetMinButtonSize() { return m_fMinButtonSize; }
	void SetMinButtonSize( float fMinButtonSize ) { m_fMinButtonSize = fMinButtonSize; RegenerateLayout(); }

	bool IsExpanded() { return m_bExpanded; }
	void ForceExpand() { ExpandToolbar(); }
	void ForceContract() { ContractToolbar(); }

protected:

	bool m_bExpanded;
	void ExpandToolbar();
	void ContractToolbar();

	bool m_bConsumingPossibleExpand;

	unsigned int m_nCurrentItem;
	std::vector< HUDItem * > m_vItems;

	void SetCurrentItem( unsigned int nItem );

	float m_fButtonSpacing;

	Orientation m_eOrientation;

	float m_fMinButtonSize;

	void InsertItem( HUDItem * pItem );
	void InsertItem_Horizontal( HUDItem * pItem );
	void InsertItem_Vertical( HUDItem * pItem );

	void NotifyToolbarChanged(bool bExpanded);

	// layout controls
	void InitializeToolbar();

	// compute bounds of toolbar
	void ComputeBounds();

	// re-set toolbar if options change
	void RegenerateLayout( );


	// open/close buttons
	rmsui::HUDButton * m_pArrowButtons[4];		// down, up, left, right
	rmsui::HUDEventHandler * m_pArrowButtonHandlers[4];		// down, up, left, right

	void UpdateArrowButtons_Vertical();
};


}   // namespace rmsui



#endif // _RMSUI_HUD_POPUP_TOOLBAR_H_