#include "StdAfx.h"
#include "Handle.h"
#include "Handle_ReferenceFrame.h"
#include "GLUtil.h"


using namespace rmsui;



bool HandleManager::Add3DTranslationHandle( IHandleTarget * pTarget )
{
	WorldTranslationTool3D * newHandle =
		new WorldTranslationTool3D(pTarget);

	m_sHandles.insert( newHandle );

	return true;
}

bool HandleManager::Add3DScaleHandle( IHandleTarget * pTarget )
{

	return false;
}
 
bool HandleManager::Add3DRotationHandle( IHandleTarget * pTarget )
{
	WorldRotationTool3D * newHandle =
		new WorldRotationTool3D(pTarget);

	m_sHandles.insert( newHandle );

	return false;
}



bool HandleManager::RemoveHandles( IHandleTarget * pTarget )
{
	// can't do this yet
	return false;
}

bool HandleManager::RemoveAllHandles()
{
	// should we do any ordering here? maybe selected ones last (so they are on top??)

	HandleSet::iterator cur(m_sHandles.begin()), end(m_sHandles.end());
	while (cur != end) {
		delete (*cur);
		++cur;
	}
	m_sHandles.clear();
	return true;
}




/*
 * handle picking support
 */


IHandle * HandleManager::SelectHandle( int screenX, int screenY, Wml::ExtendedWmlCamera & camera, 
									   bool bClearExistingSelection)
{
	if (bClearExistingSelection) {
		ClearSelectedHandles();
	}

	IHandle * pHandle = PickNearestHandle( screenX, screenY, camera);
	if (pHandle) {
		HandleSelectedChanged(pHandle, true);
	}

	return pHandle;
}


IHandle * HandleManager::UnSelectHandle( int screenX, int screenY, Wml::ExtendedWmlCamera & camera )
{
	IHandle * pHandle = PickNearestHandle( screenX, screenY, camera, true);
	if (pHandle) {
		HandleSelectedChanged(pHandle, false);
	}

	return pHandle;
}


void HandleManager::ClearSelectedHandles()
{
	HandleSet::iterator cur(m_sSelectedHandles.begin()), end(m_sSelectedHandles.end());
	while (cur != end) {
		(*cur)->SetSelected(false);
		++cur;
	}
	m_sSelectedHandles.clear();
}


IHandle * HandleManager::PickNearestHandle(int screenX, int screenY, Wml::ExtendedWmlCamera & camera, bool bSelectedOnly)
{
	// create eye ray in fixed view
	Wml::Ray3f ray;
	GenerateEyeRay(screenX, screenY, m_viewState.dFixedView_ModelViewMatrix, 
					m_viewState.dFixedView_ProjectionMatrix, ray);

	// compute view orientation matrix (doesn't change per handle)
	Wml::Matrix3f matOrientation;
	camera.GetFrameMatrix(matOrientation);
	matOrientation = matOrientation.Transpose();

	// keep track of nearest handle, intersection point, and intersection z-depth
	IHandle * pNearestHandle = NULL;
	float fNearest = Wml::Mathf::MAX_REAL;
	Wml::Vector3f ptNearest;

	HandleSet::iterator cur, end;
	cur = (bSelectedOnly) ? m_sSelectedHandles.begin() : m_sHandles.begin();
	end = (bSelectedOnly) ? m_sSelectedHandles.end() : m_sHandles.end();
	while (cur != end) {

		IHandle * pHandle = (*cur++);

		// transform fixed-view eye ray into local basis of handle
		Wml::Ray3f transformedRay;
		TransformRayIntoHandleSpace(pHandle, ray, matOrientation, transformedRay);

		// intersect ray, save hit if it's closer than closest
		float fDepth = Wml::Mathf::MAX_REAL;
		float fRayOrigin = transformedRay.Direction().Dot(transformedRay.Origin());
		Wml::Vector3f ptHit;
		if (pHandle->Intersect( transformedRay, ptHit, fDepth)) {
			if (fDepth > fRayOrigin && fDepth < fNearest) {
				fNearest = fDepth;
				ptNearest = ptHit;
				pNearestHandle = pHandle;
			}
		}

	}

	// RMS TODO this is sort of bad... ??
	if (pNearestHandle) {
		GenerateEyeRay(screenX, screenY, m_viewState.dEyeView_ModelViewMatrix, 
						m_viewState.dEyeView_ProjectionMatrix, m_targetInfo.eyeViewRay);
		TransformRayIntoHandleSpace( pNearestHandle, ray, matOrientation, m_targetInfo.fixedViewOrientedRay);
		pNearestHandle->Move(m_targetInfo, m_viewState, true);
	}

	return pNearestHandle;
}


void HandleManager::HandleSelectedChanged( IHandle * pHandle, bool bSelected )
{
	if (bSelected) {
		m_sSelectedHandles.insert(pHandle);
	} else {
		HandleSet::iterator found( m_sSelectedHandles.find(pHandle) );
		if (found != m_sSelectedHandles.end()) {
			m_sSelectedHandles.erase(found);
		}
	}
	pHandle->SetSelected(bSelected);
}


/*
 * interaction support
 */

void HandleManager::MoveSelectedHandles( int screenX, int screenY, const Wml::ExtendedWmlCamera & camera )
{
	int iWidth = camera.GetViewport()[2];
	int iHeight = camera.GetViewport()[3];

	// create initial pick rays
	GenerateEyeRay(screenX, screenY, m_viewState.dEyeView_ModelViewMatrix, 
					m_viewState.dEyeView_ProjectionMatrix, m_targetInfo.eyeViewRay);
	Wml::Ray3f fixedViewRay;
	GenerateEyeRay(screenX, screenY, m_viewState.dFixedView_ModelViewMatrix, 
					m_viewState.dFixedView_ProjectionMatrix, fixedViewRay);

	// compute view orientation matrix (doesn't change per handle)
	Wml::Matrix3f matOrientation;
	camera.GetFrameMatrix(matOrientation);
	matOrientation = matOrientation.Transpose();

	// move handles
	HandleSet::iterator cur, end;
	cur = m_sSelectedHandles.begin();
	end = m_sSelectedHandles.end();
	while (cur != end) {

		IHandle * pHandle = (*cur++);

		// transform fixed-view eye ray into local basis of handle
		TransformRayIntoHandleSpace(pHandle, fixedViewRay, matOrientation, m_targetInfo.fixedViewOrientedRay);

		// compute new position from ray
		Wml::Vector3f & newPosition = 
			pHandle->Move( m_targetInfo, m_viewState );

		// translate

		// unproject new position
	}
}



