#ifndef RMSUI_HANDLE_H
#define RMSUI_HANDLE_H


#include <WmlVector3.h>
#include <WmlRay3.h>
#include <WmlColorRGB.h>
#include <ExtendedWmlCamera.h>

#include <set>
#include <vector>
#include <map>

#include "Frame.h"

namespace rmsui
{














/**
 * 
 */
class HandleManager
{
public:
	
	~HandleManager()
	{
		RemoveAllHandles();
	}


	// add/remove handles

	bool Add3DTranslationHandle( IHandleTarget * pTarget );
	bool Add3DScaleHandle( IHandleTarget * pTarget ); 
	bool Add3DRotationHandle( IHandleTarget * pTarget );

	bool RemoveHandles( IHandleTarget * pTarget );
	bool RemoveAllHandles();

	bool GetHandlesSelected()
		{	return !m_sSelectedHandles.empty(); }



	// rendering support

	void RenderHandles( const Wml::ExtendedWmlCamera & renderCamera );


	// selection support

	IHandle * SelectHandle( int screenX, int screenY, Wml::ExtendedWmlCamera & camera, 
							bool bClearExistingSelection = true );

	IHandle * UnSelectHandle( int screenX, int screenY, Wml::ExtendedWmlCamera & camera );

	void ClearSelectedHandles();


	// interaction support

	void MoveSelectedHandles( int screenX, int screenY, const Wml::ExtendedWmlCamera & camera );


protected:

	typedef std::vector<IHandle *> HandleList;
	typedef std::set<IHandle *> HandleSet;
	typedef std::map< IHandleTarget *, HandleList > HandleTargetMap;

	//! map handle targets to their list of handles
	HandleTargetMap m_mTargets;

	//! set of all handles that exist
	HandleSet m_sHandles;

	//! set of selected handles (subset of m_sHandles)
	HandleSet m_sSelectedHandles;

/*
 * rendering support
 */
	//! current view state - opengl matrices, etc
	ViewState m_viewState;

	//! rendering support function - set new frustum so handles are fixed-size
	void SetupFixedSizeHandleView( const Wml::ExtendedWmlCamera & renderCamera );	

	/*! compute translation to handle that places it's origin at the correct position
	    in the fixed-size perspective space */
	void ComputeFixedSizeTranslation( const IHandle * pHandle,
									  Wml::Vector3f & nearPlanePoint, 
									  Wml::Vector3f & directionIntoScene );

	//! generate a ray using the given modelview and projection matrices
	void GenerateEyeRay( int nScreenX, int nScreenY, 
						 double dModelViewMatrix[16], double dProjectionMatrix[16], 
						 Wml::Ray3f & ray );

	//! transform a fixed-view space ray into the local space of a handle
	void TransformRayIntoHandleSpace( const IHandle * pHandle, const Wml::Ray3f & initialRay,
									  const Wml::Matrix3f & matViewOrientation,
									  Wml::Ray3f & destRay );
									

/*
 * picking support
 */

	void HandleSelectedChanged( IHandle * pHandle, bool bSelected );

	IHandle * PickNearestHandle(int screenX, int screenY, Wml::ExtendedWmlCamera & camera, bool bSelectedOnly = false);

/*
 * movement support
 */
	SelectedTargetInfo m_targetInfo;
};




}		// end namespace rmsui



#endif  // RMSUI_HANDLE_H