// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#pragma once

#include <set>
#include <map>
#include <limits>
#include <vector>


namespace rmsui
{

struct InputEventButtonState {
	bool bDown;
	bool bWasDown;
};

class InputEventData {
public:

	enum InputEventType {
		MouseEvent,
		NoEvent
	};

	InputEventType eType;

	// mouse data
	int nMouseX;
	int nMouseY;

	InputEventButtonState LeftButton;
	InputEventButtonState RightButton;
	InputEventButtonState MiddleButton;

	InputEventData() {
		eType = NoEvent;
		nMouseX = nMouseY = std::numeric_limits<int>::max();
		LeftButton.bDown = LeftButton.bWasDown = false;
		RightButton.bDown = RightButton.bWasDown = false;
		MiddleButton.bDown = MiddleButton.bWasDown = false;
	}

	~InputEventData() {}
};


class IInputEventListener
{
public:
	enum EventState {
		Consumed,
		Ignored,
		StartCapture,
		EndCapture
	};

	virtual ~IInputEventListener() {}

	virtual EventState ProcessInputEvent( const InputEventData & event ) = 0;
};

typedef CWnd * CurrentEventStreamKey;

class CInputEventStream
{
public:

	static void SetCurrentEventStream( CurrentEventStreamKey key );
	static CInputEventStream * Get();

	void AddListener( IInputEventListener * pListener );
	void RemoveListener( IInputEventListener * pListener ); 

	void AppendLeftMouseDownEvent( int x, int y );
	void AppendLeftMouseUpEvent( int x, int y );
	void AppendLeftMouseMoveEvent( int x, int y );

	void AppendRightMouseDownEvent( int x, int y );
	void AppendRightMouseUpEvent( int x, int y );
	void AppendRightMouseMoveEvent( int x, int y );


protected:
	CInputEventStream(void);
	~CInputEventStream(void);

	//std::set< IInputEventListener * > m_eventHandlers;
	std::vector< IInputEventListener * > m_vEventHandlers;

	void SendEvent( const InputEventData & event );

	IInputEventListener * m_pCaptureTarget;

	// list of streams
	static std::map< CurrentEventStreamKey, CInputEventStream * > s_streams;
	static CurrentEventStreamKey s_CurrentKey;
};


inline void CInputEventStream::AddListener( IInputEventListener * pListener ) 
{
	//m_eventHandlers.insert(pListener);
	m_vEventHandlers.push_back(pListener);
}

inline void CInputEventStream::RemoveListener( IInputEventListener * pListener ) 
{
	//m_eventHandlers.erase(pListener);
	size_t nCount = m_vEventHandlers.size();
	for ( unsigned int i = 0; i < nCount; ++i ) {
		if ( m_vEventHandlers[i] == pListener ) {
			for ( unsigned int j = i; j < nCount-1; ++j )
				m_vEventHandlers[j] = m_vEventHandlers[j+1];
			i = (unsigned int)nCount;
		}
	}
}



} // end namespace rmsui