// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef _RMSUI_RENDER_BUFFFER_H
#define _RMSUI_RENDER_BUFFFER_H

#include <vector>

#include <gl/gl.h>
#include <gl/glu.h>

namespace rmsui
{

/*
 * this class is an off-screen rendering buffer
 */
class RenderBuffer {
public:
	RenderBuffer();
	~RenderBuffer();

	bool Initialize(unsigned int nWidth, unsigned int nHeight);

	unsigned int Width() { return m_nWidth; }
	unsigned int Height() { return m_nHeight; }

	void EnableGLContext(bool bSaveContext);
	void DisableGLContext(bool bRestoreContext);

	// channel map [0,1,2] is RGB, [2,1,0] is BGR
	//! note: does not seem to return proper alpha...
	void GetBuffer( std::vector<unsigned char> & vBuffer, 
					const unsigned int nChannelMap[3],
					bool bWantAlpha );

	//! call glReadPixels (must call when context is valid)
	void GetGLBuffer( std::vector<unsigned char> & vBuffer,
					  bool bWantAlpha );

	void GetDepthBuffer( std::vector<float> & vBuffer, bool bScale );

protected:
	CDC * m_pCDC;
	CBitmap * m_pOldDCBitmap;
	CBitmap * m_pBitmap;
	HGLRC m_hGLRC;

	HDC m_hSavedContextDC;
	HGLRC m_hSavedContextGLRC;

	unsigned int m_nWidth;
	unsigned int m_nHeight;

	BOOL SetPixelFormat( CDC * pDC );

};


} // namespace rmsui


#endif // _RMSUI_RENDER_BUFFFER_H