// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt

#include "StdAfx.h"
#include "DebugData.h"

using namespace Wml;

#include <gl/GL.h>
#include <gl/GLU.h>

void DebugData::Clear()
{
	m_vPoints.clear();
	m_vLines.clear();
	m_vLineStrips.clear();
}

void DebugData::AddLine( const Wml::Vector2f & v1, const Wml::Vector2f & v2,
						 const Wml::ColorRGB & color, float fWidth )
{
	AddLine( Wml::Vector3f(v1.X(), v1.Y(), 0), Wml::Vector3f(v2.X(), v2.Y(), 0), color, fWidth );
}


void DebugData::AddLine( const Wml::Vector3f & v1, const Wml::Vector3f & v2,
						 const Wml::ColorRGB & color, float fWidth )
{
	Line l;
	l.vPoint1 = v1;
	l.vPoint2 = v2;
	l.color = color;
	l.fWidth = fWidth;
	m_vLines.push_back(l);
}


void DebugData::AddPoint( const Wml::Vector3f & vPos, 
						  const Wml::ColorRGB & color )
{
	Point p;
	p.vPosition = vPos;
	p.color =  color;
	m_vPoints.push_back(p);
}


void DebugData::RenderPoints()
{
	glBegin(GL_POINTS);
	size_t nPoints = m_vPoints.size();
	for ( unsigned int i = 0; i < nPoints; ++i ) {
		glColor3fv( m_vPoints[i].color );
		glVertex3fv( m_vPoints[i].vPosition );
	};
	glEnd();
}

void DebugData::RenderLines()
{
	size_t nLines = m_vLines.size();
	for ( unsigned int i = 0; i < nLines; ++i ) {
		glLineWidth( m_vLines[i].fWidth );
		glBegin(GL_LINE_STRIP);
		glColor3fv( m_vLines[i].color );
		glVertex3fv( m_vLines[i].vPoint1 );
		glVertex3fv( m_vLines[i].vPoint2 );
		glEnd();
	}

	size_t nStrips = m_vLineStrips.size();
	for ( unsigned int i = 0; i < nStrips; ++i ) {
		glLineWidth( m_vLineStrips[i].fWidth );
		glBegin(GL_LINE_STRIP);
		glColor3fv( m_vLineStrips[i].color );
		size_t nVerts = m_vLineStrips[i].vVertices.size();
		for ( unsigned int j = 0; j <  nVerts; ++j )
			glVertex3fv( m_vLineStrips[i].vVertices[j] );
		glEnd();
	}
	
}