// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"

#include "OpenGLWmlLight.h"
#include <gl/GL.h>
#include <gl/GLU.h>

using namespace Wml;
using namespace std;

WmlImplementRTTI(OpenGLWmlLight, Light);

void OpenGLWmlLight::SetupLight()
{
	unsigned int glLightNum = GL_LIGHT0 + m_nLightIndex;
	if ( On() ) {
		glEnable( glLightNum );

		// not handling spotlight, ambient

		// set up colors
		float color[4] = {0.0, 0.0, 0.0, 1.0};

		ColorRGB ambient = Intensity() * Ambient();
//		ambient.Clamp();
		color[0] = ambient.r;  color[1] = ambient.g;  color[2] = ambient.b;
		glLightfv(glLightNum, GL_AMBIENT, color);

		ColorRGB diffuse = Intensity() * Diffuse();
//		diffuse.Clamp();
		color[0] = diffuse.r;  color[1] = diffuse.g;  color[2] = diffuse.b;
		glLightfv(glLightNum, GL_DIFFUSE, color);

		ColorRGB specular = Intensity() * Specular();
//		specular.Clamp();
		color[0] = specular.r;  color[1] = specular.g;  color[2] = specular.b;
		glLightfv(glLightNum, GL_SPECULAR, color);

		// set position
		float position[4] = {Position().X(), Position().Y(), Position().Z(), 1.0};
		if (Type() == LT_DIRECTIONAL)
			position[3] = 0.0;
		glLightfv(glLightNum, GL_POSITION, position);

		if (Attenuate()) {
			glLightf(glLightNum, GL_CONSTANT_ATTENUATION, Constant());
			glLightf(glLightNum, GL_LINEAR_ATTENUATION, Linear());
			glLightf(glLightNum, GL_QUADRATIC_ATTENUATION, Quadratic());
		}

	} else {
		glDisable(GL_LIGHT0 + m_nLightIndex );
	}
}