// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"

#include "WmlExtDistLin3Sph3.h"
#include "WmlDistVec3Lin3.h"
#include "WmlIntrLin3Sph3.h"
using namespace Wml;

template <class Real>
Real Wml::SqrDistance (const Line3<Real>& rkLine,
					   const Sphere3<Real>& rkSphere, Vector3<Real>* pkLineClosest,
					   Vector3<Real>* pkSphereClosest)
{
	Real rDistance = Distance(rkLine, rkSphere, pkLineClosest, pkSphereClosest);
	return rDistance * rDistance;
}


template <class Real>
Real Wml::Distance (const Line3<Real>& rkLine,
					const Sphere3<Real>& rkSphere, Vector3<Real>* pkLineClosest,
					Vector3<Real>* pkSphereClosest)
{
	Real fLineParam;
	Real rCentreDist = Wml::Distance( rkSphere.Center(), rkLine, &fLineParam);

	if (rCentreDist < 0) {
		if (pkLineClosest || pkSphereClosest) {
			Vector3<Real> vIntersections[2];
			int iQuantity;
			FindIntersection( rkLine, rkSphere, iQuantity, vIntersections );
			*pkLineClosest = *pkSphereClosest = vIntersections[0];
		}
		return 0.0;

	} else {
		if (pkSphereClosest) {
			Vector3<Real> ptLine( rkLine.Origin() + rkLine.Direction() * fLineParam );
			Vector3<Real> vDir( ptLine - rkSphere.Center() );
			vDir.Normalize();
			*pkSphereClosest = rkSphere.Center() + vDir * rkSphere.Radius();
			if (pkLineClosest) {
				*pkLineClosest = ptLine;
			}
		} else if (pkLineClosest) {
			*pkLineClosest = Vector3<Real>( rkLine.Origin() + rkLine.Direction() * fLineParam );
		}
		return 	rCentreDist - rkSphere.Radius();
	}
}




namespace Wml
{
template WMLEXT_ITEM float SqrDistance<float> (const Line3<float>&,
    const Sphere3<float>&, Vector3<float>*, Vector3<float>*);
template WMLEXT_ITEM float Distance<float> (const Line3<float>&,
    const Sphere3<float>&, Vector3<float>*, Vector3<float>*);

template WMLEXT_ITEM double SqrDistance<double> (const Line3<double>&,
    const Sphere3<double>&, Vector3<double>*, Vector3<double>*);
template WMLEXT_ITEM double Distance<double> (const Line3<double>&,
    const Sphere3<double>&, Vector3<double>*, Vector3<double>*);
}
