// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"

#include "WmlExtIntrPln3Lin3.h"
using namespace Wml;

template <class Real>
bool Wml::FindIntersection (const Plane3<Real>& rkPlane,
    const Ray3<Real>& rkRay, int& riQuantity, Vector3<Real> akP[2], bool bUnitNormal)
{
	Wml::Vector3<Real> vPlaneNormal( rkPlane.GetNormal() );
	if ( ! bUnitNormal )
		vPlaneNormal.Normalize();

	Real fPlaneD = rkPlane.GetConstant();

	Real fDdotN = vPlaneNormal.Dot( rkRay.Direction() );
	if (fabs(fDdotN) < Wml::Mathf::EPSILON) {
		riQuantity = 0;
		return false;
	}

	Real fT = (fPlaneD - vPlaneNormal.Dot( rkRay.Origin()) ) / fDdotN;
	Wml::Vector3<Real> ptPlane( rkRay.Origin() + rkRay.Direction() * fT );	

	riQuantity = 1;
	akP[0] = ptPlane;
	return true;
}





namespace Wml
{
template WMLEXT_ITEM bool FindIntersection<float> (const Plane3<float>& rkPlane,
    const Ray3<float>& rkRay, int& riQuantity, Vector3<float> akP[2], bool bUnitNormal);

template WMLEXT_ITEM bool FindIntersection<double> (const Plane3<double>& rkPlane,
    const Ray3<double>& rkRay, int& riQuantity, Vector3<double> akP[2], bool bUnitNormal);
}
