// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"
#include "WmlPolynomialRootsExt.h"

#include <WmlEigen.h>
#include "WmlLinearSystemExt.h"

#include "rmsdebug.h"
#include <limits>

using namespace Wml;


template <class Real>
void PolynomialRootsExt<Real>::MakeHessenbergMatrix( Wml::Polynomial1<Real> & Poly, Wml::GMatrix<Real>& mHessenberg )
{
	int nDegree = Poly.GetDegree();
	int nRows = nDegree;

	mHessenberg.SetSize(nRows,nRows);
	for ( int i = 0; i < nRows; ++i ) {
		mHessenberg[0][i] = -Poly[ nDegree - (i+1) ] / Poly[ nDegree ];
		for ( int j = 1; j < nRows; ++j )
			mHessenberg[j][i] = (Real)0;
		if ( i != nRows-1 )
			mHessenberg[i+1][i] = (Real)1;
	}
}

template <class Real>
bool PolynomialRootsExt<Real>::EigenRoots( Wml::Polynomial1<Real> & Poly, Real * pRoots, int & nCount )
{
	Wml::GMatrix<Real> mHess;
	MakeHessenbergMatrix( Poly, mHess );

	Wml::LinearSystemExt<Real>::Balance(mHess);

	std::vector<Wml::Vector2<Real> > vRoots( Poly.GetDegree() );

	bool bResult = Wml::LinearSystemExt<Real>::QREigenValues(mHess, &vRoots[0]);
	if (!bResult)
		return false;
	nCount = 0;
	int nDegree = Poly.GetDegree();
	for ( int i = 0; i < nDegree; ++i ) {
//		if ( vRoots[i].Y() == (Real)0 ) {
		if ( fabs(vRoots[i].Y()) < (Real)0.0001 ) {
			pRoots[nCount] = vRoots[i].X();
			++nCount;
		}
	}

	return true;
}


namespace Wml {

template class WMLEXT_ITEM PolynomialRootsExt<float>;
template class WMLEXT_ITEM PolynomialRootsExt<double>;

}