// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef __RMS_SERIALIZER_H__
#define __RMS_SERIALIZER_H__

#include "WmlExt.h"
#include "WmlSerializerRecord.h"

namespace Wml {


class WMLEXT_ITEM SerializerRecordReader
{
public:
	virtual bool ReadRecord( Wml::SerializerRecord * pRecord ) = 0;
};


class WMLEXT_ITEM Serializer
{
public:
	Serializer();
	~Serializer();

	// writing
	SerializerRecord * CreateRecord( SerializerRecord::RecordType nType );
	SerializerRecord * CreateRecord( SerializerRecord::RecordID nID, SerializerRecord::RecordType nType );
	
	
	// reading
	void AddReader( SerializerRecordReader * pReader ) { m_vReaders.push_back( pReader ); }
	
	void RestoreRecords();


	SerializerRecord * FindRecord( SerializerRecord::RecordID nID );
	
	void AddItem( SerializerRecord::RecordID nID, void * pItem );
	void * FindItem( SerializerRecord::RecordID nID );


	// File I/O
	bool Write( const char * pFilename );
	bool Read( const char * pFilename );

protected:

	std::vector< SerializerRecord *> m_recordList;		// we want to write records in order...
	std::map< SerializerRecord::RecordID, SerializerRecord * > m_recordMap;

	std::vector< SerializerRecordReader * > m_vReaders;

	// this map lets us find "objects" by RecordID...
	std::map< SerializerRecord::RecordID, void * > m_itemMap;
};


}	// namespace Wml



#endif // __RMS_SERIALIZER_H__



