// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef __RMS_SERIALIZER_RECORD_H__
#define __RMS_SERIALIZER_RECORD_H__

#include "WmlExt.h"
#include "Frame.h"

#include <WmlVector3.h>
#include <WmlAxisAlignedBox2.h>
#include <map>
#include <vector>
#include <string>

namespace Wml {

class WMLEXT_ITEM SerializerRecord
{
public:
	typedef unsigned int RecordID;
	typedef unsigned int RecordType;

	static const RecordType UnknownType;

	SerializerRecord( RecordType nType );
	SerializerRecord( RecordID nID, RecordType nType );
	~SerializerRecord();

	RecordID GetRecordID() { return m_recordID; }
	RecordType GetRecordType() { return m_recordType; }

	void SetValue( const std::string & key, unsigned int nValue );
	void SetValue( const std::string & key, int nValue );
	void SetValue( const std::string & key, float fValue );
	void SetValue( const std::string & key, double dValue );	

	void SetValue( const std::string & key, const Vector3f & vValue );
	void SetValue( const std::string & key, const Vector2f & vValue );
	void SetValue( const std::string & key, const Frame3f & vFrame );
	void SetValue( const std::string & key, AxisAlignedBox2f & vBox );

	void SetValue( const std::string & key, const std::vector<float> & vValue );
	void SetValue( const std::string & key, const std::vector<RecordID> & vValue );
	void SetValue( const std::string & key, const std::vector<Wml::Vector3f> & vValue );
	void SetValue( const std::string & key, const std::vector<Wml::Vector2f> & vValue );

	void SetValue( const std::string & key, const std::string & value );


	bool GetValue( const std::string & key, unsigned int & nValue, unsigned int nDefault );
	bool GetValue( const std::string & key, int & nValue, int nDefault );
	bool GetValue( const std::string & key, float & fValue, float fDefault );
	bool GetValue( const std::string & key, double & dValue, double dDefault );

	bool GetValue( const std::string & key, Vector3f & vValue, const Vector3f & vDefault );
	bool GetValue( const std::string & key, Vector2f & vValue, const Vector2f & vDefault );
	bool GetValue( const std::string & key, Frame3f & vValue, const Frame3f & vDefault );
	bool GetValue( const std::string & key, AxisAlignedBox2f & vBox, const AxisAlignedBox2f & vDefault );

	bool GetValue( const std::string & key, std::vector<float> & vValue );
	bool GetValue( const std::string & key, std::vector<RecordID> & vValue );
	bool GetValue( const std::string & key, std::vector<Wml::Vector3f> & vValue );
	bool GetValue( const std::string & key, std::vector<Wml::Vector2f> & vValue );

	bool GetValue( const std::string & key, std::string & value );

	bool GetBytes( std::vector<unsigned char> & vBytes );
	bool SetBytes( const std::vector<unsigned char> & vBytes );

	// returns unused record ID...
	static RecordID GenUniqueRecordID();

	static void GetEmptyRecordBytes( std::vector< unsigned char > & vBytes );

protected:
	RecordID m_recordID;
	RecordType m_recordType;
	std::map< std::string, std::vector<unsigned char> > m_values;

	static RecordID s_recordIDGen;
};


}	// namespace Wml



#endif // __RMS_SERIALIZER_RECORD_H__



